@@ -63,7 +63,7 @@ def escape(term):
     return str(term).replace("'", "''")
 
 
-def search(url, query, language="ADQL", maxrec=None, uploads=None, **keywords):
+def search(url, query, *, language="ADQL", maxrec=None, uploads=None, **keywords):
     """
     submit a Table Access query that returns rows matching the criteria given.
 
@@ -107,7 +107,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
     _tables = None
     _examples = None
 
-    def __init__(self, baseurl, session=None):
+    def __init__(self, baseurl, *, capability_description=None, session=None):
         """
         instantiate a Table Access Protocol service
 
@@ -118,7 +118,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         session : object
            optional session to use for network requests
         """
-        super().__init__(baseurl, session=session)
+        super().__init__(baseurl, session=session, capability_description=capability_description)
 
         # Check if the session has an update_from_capabilities attribute.
         # This means that the session is aware of IVOA capabilities,
@@ -247,7 +247,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         return self.get_tap_capability().uploadmethods
 
     def run_sync(
-            self, query, language="ADQL", maxrec=None, uploads=None,
+            self, query, *, language="ADQL", maxrec=None, uploads=None,
             **keywords):
         """
         runs sync query and returns its result
@@ -281,7 +281,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
     search = run_sync
 
     def run_async(
-            self, query, language="ADQL", maxrec=None, uploads=None,
+            self, query, *, language="ADQL", maxrec=None, uploads=None,
             **keywords):
         """
         runs async query and returns its result
@@ -328,7 +328,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         return result
 
     def submit_job(
-            self, query, language="ADQL", maxrec=None, uploads=None,
+            self, query, *, language="ADQL", maxrec=None, uploads=None,
             **keywords):
         """
         submit a async query without starting it and returns a AsyncTAPJob
@@ -360,7 +360,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             session=self._session, **keywords)
 
     def create_query(
-            self, query=None, mode="sync", language="ADQL", maxrec=None,
+            self, query=None, *, mode="sync", language="ADQL", maxrec=None,
             uploads=None, **keywords):
         """
         create a query object that constraints can be added to and then
@@ -409,7 +409,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
                                     decode_content=True)
         return uws.parse_job(response.raw.read)
 
-    def get_job_list(self, phases=None, after=None, last=None,
+    def get_job_list(self, *, phases=None, after=None, last=None,
                      short_description=True):
         """
         lists jobs that the caller can see in the current security context.
@@ -485,7 +485,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             print()
 
     @prototype_feature('cadc-tb-upload')
-    def create_table(self, name, definition, format='VOSITable'):
+    def create_table(self, name, definition, *, format='VOSITable'):
         """
         Creates a table in the catalog service.
 
@@ -534,7 +534,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         response.raise_for_status()
 
     @prototype_feature('cadc-tb-upload')
-    def load_table(self, name, source, format='tsv'):
+    def load_table(self, name, source, *, format='tsv'):
         """
         Loads content to a table
 
@@ -565,7 +565,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         response.raise_for_status()
 
     @prototype_feature('cadc-tb-upload')
-    def create_index(self, table_name, column_name, unique=False):
+    def create_index(self, table_name, column_name, *, unique=False):
         """
         Creates a table index in the catalog service.
 
@@ -615,7 +615,7 @@ class AsyncTAPJob:
 
     @classmethod
     def create(
-            cls, baseurl, query, language="ADQL", maxrec=None, uploads=None,
+            cls, baseurl, query, *, language="ADQL", maxrec=None, uploads=None,
             session=None, **keywords):
         """
         creates a async tap job on the server under ``baseurl``
@@ -643,7 +643,7 @@ class AsyncTAPJob:
         job = cls(response.url, session=session)
         return job
 
-    def __init__(self, url, session=None):
+    def __init__(self, url, *, session=None):
         """
         initialize the job object with the given url and fetch remote values
 
@@ -920,7 +920,7 @@ class AsyncTAPJob:
 
         return self
 
-    def wait(self, phases=None, timeout=600.):
+    def wait(self, *, phases=None, timeout=600.):
         """
         waits for the job to reach the given phases.
 
@@ -1030,7 +1030,7 @@ class TAPQuery(DALQuery):
     """
 
     def __init__(
-            self, baseurl, query, mode="sync", language="ADQL", maxrec=None,
+            self, baseurl, query, *, mode="sync", language="ADQL", maxrec=None,
             uploads=None, session=None, **keywords):
         """
         initialize the query object with the given parameters
@@ -1081,7 +1081,7 @@ class TAPQuery(DALQuery):
         """
         return '{baseurl}/{mode}'.format(baseurl=self.baseurl, mode=self._mode)
 
-    def execute_stream(self, post=False):
+    def execute_stream(self, *, post=False):
         """
         submit the query and return the raw VOTable XML as a file stream
 
@@ -1116,7 +1116,7 @@ class TAPQuery(DALQuery):
         """
         return TAPResults(self.execute_votable(), url=self.queryurl, session=self._session)
 
-    def submit(self, post=False):
+    def submit(self, *, post=False):
         """
         Does the request part of the TAP query.
         This function is separated from response parsing because async queries
