@@ -52,7 +52,7 @@ def _capabilities(mocker, scope="session"):
 
 
 @pytest.fixture(name='regtap_pulsar_distance_response')
-def _regtap_pulsar_distance_response(mocker, scope="session"):
+def _regtap_pulsar_distance_response(mocker, capabilities, scope="session"):
     with mocker.register_uri(
         'POST', REGISTRY_BASEURL + '/sync',
             content=get_pkg_data_contents('data/regtap.xml')) as matcher:
@@ -60,7 +60,7 @@ def _regtap_pulsar_distance_response(mocker, scope="session"):
 
 
 @pytest.fixture()
-def keywords_fixture(mocker, scope="session"):
+def keywords_fixture(mocker, capabilities, scope="session"):
     def keywordstest_callback(request, context):
         data = dict(parse_qsl(request.body))
         query = data['QUERY']
@@ -83,12 +83,13 @@ def keywords_fixture(mocker, scope="session"):
 
 
 @pytest.fixture()
-def single_keyword_fixture(mocker, scope="session"):
+def single_keyword_fixture(mocker, capabilities, scope="session"):
     def keywordstest_callback(request, context):
         data = dict(parse_qsl(request.body))
         query = data['QUERY']
 
-        assert "OR  rr.res_subject.res_subject ILIKE '%single%'" in query
+        assert (" UNION ALL SELECT DISTINCT ivoid FROM rr.res_subject WHERE "
+                "rr.res_subject.res_subject ILIKE '%single%'") in query
         assert "1=ivo_hasword(res_description, 'single') " in query
         assert "1=ivo_hasword(res_title, 'single')" in query
 
@@ -102,7 +103,7 @@ def single_keyword_fixture(mocker, scope="session"):
 
 
 @pytest.fixture()
-def servicetype_fixture(mocker, scope="session"):
+def servicetype_fixture(mocker, capabilities, scope="session"):
     def servicetypetest_callback(request, context):
         data = dict(parse_qsl(request.body))
         query = data['QUERY']
@@ -123,7 +124,7 @@ def servicetype_fixture(mocker, scope="session"):
 
 
 @pytest.fixture()
-def waveband_fixture(mocker, scope="session"):
+def waveband_fixture(mocker, capabilities, scope="session"):
     def wavebandtest_callback(request, content):
         data = dict(parse_qsl(request.body))
         query = data['QUERY']
@@ -140,13 +141,13 @@ def waveband_fixture(mocker, scope="session"):
 
 
 @pytest.fixture()
-def datamodel_fixture(mocker, scope="session"):
+def datamodel_fixture(mocker, capabilities, scope="session"):
     def datamodeltest_callback(request, content):
         data = dict(parse_qsl(request.body))
         query = data['QUERY']
 
         assert (
-            "(detail_xpath = '/capability/dataModel/@ivo-id'" in query)
+            "detail_xpath = '/capability/dataModel/@ivo-id'" in query)
 
         assert (
             "ivo_nocasematch(detail_value, 'ivo://ivoa.net/std/obscore%'))"
@@ -162,7 +163,7 @@ def datamodel_fixture(mocker, scope="session"):
 
 
 @pytest.fixture()
-def aux_fixture(mocker, scope="session"):
+def aux_fixture(mocker, capabilities, scope="session"):
     def auxtest_callback(request, context):
         data = dict(parse_qsl(request.body))
         query = data['QUERY']
@@ -179,7 +180,7 @@ def aux_fixture(mocker, scope="session"):
 
 
 @pytest.fixture(name='multi_interface_fixture')
-def _multi_interface_fixture(mocker, scope="session"):
+def _multi_interface_fixture(mocker, capabilities, scope="session"):
     # to update this, run
     # import requests
     # from pyvo.registry import regtap
@@ -204,23 +205,25 @@ def _flash_service(multi_interface_fixture, scope="session"):
 
 class TestInterfaceClass:
     def test_basic(self):
-        intf = regtap.Interface("http://example.org", "", "", "")
+        intf = regtap.Interface("http://example.org")
         assert intf.access_url == "http://example.org"
         assert intf.standard_id is None
         assert intf.type is None
         assert intf.role is None
         assert intf.is_standard is False
         assert not intf.is_vosi
+        assert intf.capability_description is None
 
     def test_repr(self):
+        description = "An example description."
         intf = regtap.Interface("http://example.org", standard_id="ivo://gavo/std/a",
-                                intf_type="vs:paramhttp", intf_role="std")
-        assert (repr(intf) == "Interface('http://example.org',"
-                " standard_id='ivo://gavo/std/a', intf_type='vs:paramhttp', intf_role='std')")
-        intf = regtap.Interface("http://example.org", standard_id="ivo://gavo/std/a",
-                                intf_type=None, intf_role=None)
-        assert repr(intf) == ("Interface('http://example.org',"
-                              " standard_id='ivo://gavo/std/a', intf_type=None, intf_role=None)")
+                                intf_type="vs:paramhttp", intf_role="std",
+                                capability_description=description)
+        assert (repr(intf) == "Interface(type='a', description='An example description.',"
+                " url='http://example.org')")
+        intf = regtap.Interface("http://example.org", capability_description=description)
+        assert repr(intf) == ("Interface(type=None, description='An example description.',"
+                " url='http://example.org')")
 
     def test_unknown_standard(self):
         intf = regtap.Interface("http://example.org", standard_id="ivo://gavo/std/a",
@@ -347,13 +350,15 @@ def get_regtap_results(**kwargs):
 def test_spatial():
     assert (rtcons.keywords_to_constraints({
             "spatial": (23, -40)})[0].get_search_condition(FAKE_GAVO)
-            == "1 = CONTAINS(MOC(6, POINT(23, -40)), coverage)")
+            == "ivoid IN (SELECT DISTINCT ivoid FROM rr.stc_spatial"
+            " WHERE 1 = CONTAINS(MOC(6, POINT(23, -40)), coverage))")
 
 
 def test_spectral():
     assert (rtcons.keywords_to_constraints({
             "spectral": (1e-17, 2e-17)})[0].get_search_condition(FAKE_GAVO)
-            == "1 = ivo_interval_overlaps(spectral_start, spectral_end, 1e-17, 2e-17)")
+            == "ivoid IN (SELECT DISTINCT ivoid FROM rr.stc_spectral WHERE"
+            " 1 = ivo_interval_overlaps(spectral_start, spectral_end, 1e-17, 2e-17))")
 
 
 def test_to_table(multi_interface_fixture, capabilities):
@@ -457,11 +462,14 @@ class TestInterfaceSelection:
                                       " using Servicetype.")
 
     def test_get_web_interface(self, flash_service):
-        svc = flash_service.get_service("web")
+        svc = flash_service.get_service(service_type="web")
         assert isinstance(svc,
                           regtap._BrowserService)
-        assert (svc.access_url
+        assert (svc.baseurl
                 == "http://dc.zah.uni-heidelberg.de/flashheros/q/web/form")
+        assert str(svc) == ("BrowserService(baseurl : "
+                            "'http://dc.zah.uni-heidelberg.de/flashheros/q/web/form',"
+                            " description : '')")
 
         import webbrowser
         orig_open = webbrowser.open
@@ -475,25 +483,25 @@ class TestInterfaceSelection:
             webbrowser.open = orig_open
 
     def test_get_aux_interface(self, flash_service):
-        svc = flash_service.get_service("tap#aux")
+        svc = flash_service.get_service(service_type="tap#aux")
         assert (svc._baseurl
                 == "http://dc.zah.uni-heidelberg.de/tap")
 
     def test_get_aux_as_main(self, flash_service):
-        assert (flash_service.get_service("tap")._baseurl
+        assert (flash_service.get_service(service_type="tap")._baseurl
                 == "http://dc.zah.uni-heidelberg.de/tap")
 
     def test_get__main_from_aux(self, flash_service):
-        assert (flash_service.get_service("tap")._baseurl
+        assert (flash_service.get_service(service_type="tap")._baseurl
                 == "http://dc.zah.uni-heidelberg.de/tap")
 
     def test_get_by_alias(self, flash_service):
-        assert (flash_service.get_service("spectrum")._baseurl
+        assert (flash_service.get_service(service_type="spectrum")._baseurl
                 == "http://dc.zah.uni-heidelberg.de/fhssa?")
 
     def test_get_unsupported_standard(self, flash_service):
         with pytest.raises(ValueError) as excinfo:
-            flash_service.get_service("soda#sync-1.0")
+            flash_service.get_service(service_type="soda#sync-1.0")
 
         assert str(excinfo.value) == (
             "PyVO has no support for interfaces with standard id"
@@ -501,20 +509,15 @@ class TestInterfaceSelection:
 
     def test_get_nonexisting_standard(self, flash_service):
         with pytest.raises(ValueError) as excinfo:
-            flash_service.get_service("http://nonsense#fancy")
+            flash_service.get_service(service_type="http://nonsense#fancy")
 
         assert str(excinfo.value) == (
             "No matching interface.")
 
     def test_unconstrained(self, flash_service):
-        with pytest.raises(ValueError) as excinfo:
+        with pytest.raises(ValueError, match="Multiple matching interfaces found.*"):
             flash_service.get_service(lax=False)
 
-        assert str(excinfo.value) == (
-            "Multiple matching interfaces found.  Perhaps pass in"
-            " service_type or use a Servicetype constrain in the"
-            " registry.search?  Or use lax=True?")
-
     def test_interface_without_role(self):
         # There's an ugly corner case in our array simulation for
         # capabilities and interfaces: if there's a single untyped
@@ -534,9 +537,20 @@ class TestInterfaceSelection:
         # get_service still won't work because it needs a paramhttp
         # interface (and a role="std").
         with pytest.raises(ValueError) as excinfo:
-            resource.get_service('tap')
+            resource.get_service(service_type='tap')
         assert (str(excinfo.value) == "No matching interface.")
 
+    def test_get_service_by_keyword(self):
+        rec = _makeRegistryRecord(
+            access_urls=["http://example1.org/tap", "http://example2.org/tap",
+                         "http://example3.org/tap"],
+            standard_ids=["ivo://ivoa.net/std/tap"] * 3,
+            intf_types=["vs:paramhttp"] * 3,
+            intf_roles=["std"] * 3,
+            cap_descriptions=["Example 1 TAP", "Example 2 TAP"])  # no description for the third one
+        service = rec.get_service(service_type="tap", keyword="Example 2")
+        assert service.capability_description == "Example 2 TAP"
+
     def test_sia2_query(self):
         rec = _makeRegistryRecord(
             access_urls=["http://sia2.example.com", "http://sia.example.com"],
@@ -544,10 +558,11 @@ class TestInterfaceSelection:
                 "ivo://ivoa.net/std/sia#query-2.0",
                 "ivo://ivoa.net/std/sia"],
             intf_roles=["std"] * 2,
-            intf_types=["vs:paramhttp"] * 2)
+            intf_types=["vs:paramhttp"] * 2,
+            cap_descriptions=["A mock SIA2 Service."] * 2)
         assert rec.access_modes() == {"sia", "sia2"}
-        assert rec.get_interface("sia2").access_url == 'http://sia2.example.com'
-        assert rec.get_interface("sia").access_url == 'http://sia.example.com'
+        assert rec.get_interface(service_type="sia2").access_url == 'http://sia2.example.com'
+        assert rec.get_interface(service_type="sia").access_url == 'http://sia.example.com'
 
     def test_sia2_aux(self):
         rec = _makeRegistryRecord(
@@ -556,10 +571,11 @@ class TestInterfaceSelection:
                 "ivo://ivoa.net/std/sia#query-aux-2.0",
                 "ivo://ivoa.net/std/sia#aux"],
             intf_roles=["std"] * 2,
-            intf_types=["vs:paramhttp"] * 2)
+            intf_types=["vs:paramhttp"] * 2,
+            cap_descriptions=["A mock service."] * 2)
         assert rec.access_modes() == {"sia#aux", "sia2#aux"}
-        assert rec.get_interface("sia2").access_url == 'http://sia2.example.com'
-        assert rec.get_interface("sia").access_url == 'http://sia.example.com'
+        assert rec.get_interface(service_type="sia2").access_url == 'http://sia2.example.com'
+        assert rec.get_interface(service_type="sia").access_url == 'http://sia.example.com'
 
     def test_non_standard_interface(self):
         intf = regtap.Interface("http://url", standard_id="", intf_type="", intf_role="")
@@ -577,6 +593,24 @@ class TestInterfaceSelection:
         assert str(excinfo.value) == (
             "Resource ivo://pyvo/test_regtap.py is not a searchable service")
 
+    def test_list_interfaces(self):
+        rec = _makeRegistryRecord(
+            access_urls=["http://sia.example.com", "http://sia.example.com",
+                         "http://tap.example.com", "http://vosi.example.com",
+                         "http://website.com"
+                         ],
+            standard_ids=["ivo://ivoa.net/std/sia#aux",
+                          "ivo://ivoa.net/std/sia#aux",
+                          "ivo://ivoa.net/std/tap",
+                          "ivo://ivoa.net/std/vosi", ""],
+            intf_roles=["std"] * 4 + ["non standard"],
+            intf_types=["vs:paramhttp"] * 4 + ["vr:webbrowser"],
+            cap_descriptions=["A mock service."] * 5)
+        # webpages should be there, but not vosi
+        assert len(rec.list_interfaces()) == 4
+        # only sia ones
+        assert len(rec.list_interfaces("sia")) == 2
+
 
 class _FakeResult:
     """A fake class just sufficient for giving dal.query.Record enough
@@ -608,7 +642,8 @@ def _makeRegistryRecord(**overrides):
         "standard_ids": "",
         "intf_types": "",
         "intf_roles": "",
-        "ivoid": "ivo://pyvo/test_regtap.py"
+        "cap_descriptions": "",
+        "ivoid": "ivo://pyvo/test_regtap.py",
     }
     defaults.update(overrides)
     return regtap.RegistryResource(_FakeResult(defaults), 0)
@@ -625,22 +660,18 @@ class TestInterfaceRejection:
             standard_ids=["ivo://ivoa.net/std/tap"] * 2,
             intf_types=["vs:paramhttp"] * 2,
             intf_roles=["std"] * 2)
-        with pytest.raises(ValueError) as excinfo:
-            rsc.get_service("tap", lax=False)
-
-        assert str(excinfo.value) == (
-            "Multiple matching interfaces found.  Perhaps pass in"
-            " service_type or use a Servicetype constrain in the"
-            " registry.search?  Or use lax=True?")
+        with pytest.raises(ValueError, match="Multiple matching interfaces found.*"):
+            rsc.get_service(service_type="tap", lax=False)
 
     def test_nonunique_lax(self):
         rsc = _makeRegistryRecord(
             access_urls=["http://a", "http://b"],
             standard_ids=["ivo://ivoa.net/std/tap"] * 2,
             intf_types=["vs:paramhttp"] * 2,
-            intf_roles=["std"] * 2)
+            intf_roles=["std"] * 2,
+            cap_descriptions=["Test TAP service."] * 2)
 
-        assert (rsc.get_service("tap")._baseurl
+        assert (rsc.get_service(service_type="tap", lax=True)._baseurl
                 == "http://a")
 
     def test_nonstd_ignored(self):
@@ -649,7 +680,7 @@ class TestInterfaceRejection:
             standard_ids=["ivo://ivoa.net/std/tap"] * 2,
             intf_types=["vs:paramhttp"] * 2,
             intf_roles=["std", ""])
-        assert (rsc.get_service("tap", lax=False)._baseurl
+        assert (rsc.get_service(service_type="tap", lax=False)._baseurl
                 == "http://a")
 
     def test_select_single_matching_service(self):
@@ -734,7 +765,6 @@ class TestExtraResourceMethods:
             intf_roles=["std"])
         assert rsc.standard_id == "ivo://ivoa.net/std/tap"
 
-    @pytest.mark.remote_data
     def test_describe_multi(self, flash_service):
         out = io.StringIO()
         flash_service.describe(verbose=True, file=out)
@@ -743,13 +773,15 @@ class TestExtraResourceMethods:
         assert "Flash/Heros SSAP" in output
         assert ("Access modes: datalink#links-1.1, soda#sync-1.0,"
                 " ssa, tap#aux, web" in output)
-        assert "Multi-capability service" in output
+        assert "- webpage: http://dc.zah.uni-heidelberg.de/flashheros/q/web/form" in output
+        assert "- tap#aux: http://dc.zah.uni-heidelberg.de/tap" in output
+        # datalink, soda and vosi are not printed here
+        assert "- datalink" not in output
         assert "Source: 1996A&A...312..539S" in output
         assert "Authors: Wolf" in output
         assert "Alternative identifier(s): doi:10.21938/" in output
         assert "More info: http://dc.zah" in output
 
-    @pytest.mark.remote_data
     def test_describe_long_authors_list(self):
         """Check that long list of authors use et al.."""
         rsc = _makeRegistryRecord(
@@ -767,7 +799,6 @@ class TestExtraResourceMethods:
         # output should cut at 5 authors
         assert "Authors: a, a, a, a, a et al." in output
 
-    @pytest.mark.remote_data
     def test_describe_long_author_name(self):
         """Check that long author names are truncated."""
         rsc = _makeRegistryRecord(
@@ -783,7 +814,7 @@ class TestExtraResourceMethods:
         rsc.describe(verbose=True, file=out)
         output = out.getvalue()
         # should cut the long author name at 70 characters
-        assert f"Authors: {'a'*70}..." in output
+        assert f"Authors: {'a' * 70}..." in output
 
     def test_no_access_url(self):
         rsc = _makeRegistryRecord(
@@ -866,6 +897,8 @@ class TestGetTables:
         assert getcol("access_url").datatype.content == "char"
         assert getcol("access_url").datatype.arraysize == "*"
 
+        assert getcol("s_region").datatype._extendedtype == "adql:region"
+
         assert getcol("access_format").ucd == 'meta.code.mime'
 
         assert getcol("em_min").unit == "m"
